@@ -1,9 +1,10 @@
-from functools import update_wrapper, wraps
+from functools import update_wrapper, wraps, partial
 from multiprocessing import Process
 from typing import List
 import copy
 # noinspection PyUnresolvedReferences
-import apscheduler
+import nb_log
+
 from function_scheduling_distributed_framework.set_frame_config import patch_frame_config, show_frame_config
 # import frame_config
 from function_scheduling_distributed_framework.consumers.base_consumer import ExceptionForRequeue, ExceptionForRetry, \
@@ -15,6 +16,9 @@ from function_scheduling_distributed_framework.factories.consumer_factory import
 from function_scheduling_distributed_framework.utils import nb_print, patch_print, LogManager, get_logger, LoggerMixin
 from function_scheduling_distributed_framework.timing_job import fsdf_background_scheduler, timing_publish_deco
 
+# 有的包默认没加handlers，原始的日志不漂亮且不可跳转不知道哪里发生的。这里把warnning级别以上的日志默认加上handlers。
+nb_log.get_logger(name=None, log_level_int=30, log_filename='pywarning.log')
+
 
 class BrokerEnum:
     RABBITMQ_AMQPSTORM = 0  # 使用 amqpstorm 包操作rabbitmq  作为 分布式消息队列，支持消费确认.推荐这个。
@@ -62,7 +66,9 @@ class BrokerEnum:
     """
     MQTT = 17
 
-    HTTPSQS = 18
+    HTTPSQS = 18  # httpsqs，基于http协议操作
+
+    PULSAR = 20  # 最有潜力的下一代分布式消息系统。5年后会同时取代rabbitmq和kafka。
 
 
 class ConcurrentModeEnum:
@@ -73,6 +79,62 @@ class ConcurrentModeEnum:
     SINGLE_THREAD = 5
 
 
+class IdeAutoCompleteHelper(LoggerMixin):
+    """
+    为了被装饰的消费函数的敲代码时候的被pycharm自动补全而写的类。
+    """
+
+    def __init__(self, consuming_func_decorated: callable):
+        """
+        :param consuming_func_decorated:   传入被task_deco装饰的函数
+
+        此框架非常非常注重，公有函数、方法、类 的名字和入参在ide开发环境下面的自动提示补全效果，如果不是为了这一点，框架能减少很多重复地方。
+        此类是防止用户调用打错字母或者不知道怎么敲代码不知道有哪些入参。所以才有这个类。
+
+        这个类是个补全类，能够使pycharm自动补全方法名字和入参。可以用，可以不用，用了后在pycharm里面补全效果会起作用。
+
+
+       from function_scheduling_distributed_framework import task_deco, IdeAutoCompleteHelper
+
+       @task_deco('queue_test_f01', qps=2, broker_kind=3)
+       def f(a, b):
+           print(f'{a} + {b} = {a + b}')
+
+
+       if __name__ == '__main__':
+           f(1000, 2000)
+           IdeAutoCompleteHelper(f).clear()  # f.clear()
+           for i in range(100, 200):
+               f.pub(dict(a=i, b=i * 2))  # f.sub方法是强行用元编程加到f上去的，是运行时状态，pycharm只能补全非运行时态的静态东西。
+               IdeAutoCompleteHelper(f).pub({'a': i * 3, 'b': i * 4})  # 和上面的发布等效，但可以自动补全方法名字和入参。
+               f.push(a=i, b=i * 2)
+               IdeAutoCompleteHelper(f).delay(i * 3,  i * 4)
+
+           IdeAutoCompleteHelper(f).start_consuming_message()  # 和 f.consume()等效
+
+        """
+        self.is_decorated_as_consume_function = consuming_func_decorated.is_decorated_as_consume_function
+
+        self.consuming_func_decorated = consuming_func_decorated
+
+        self.consumer = consuming_func_decorated.consumer  # type: AbstractConsumer
+        self.start_consuming_message = self.consume = self.start = self.consumer.start_consuming_message
+
+        self.publisher = consuming_func_decorated.publisher  # type: AbstractPublisher
+        self.publish = self.pub = self.publisher.publish  # type: AbstractPublisher.publish
+        self.push = self.delay = self.publisher.push  # type: AbstractPublisher.push
+        self.clear = self.clear_queue = self.publisher.clear  # type: AbstractPublisher.clear
+
+        # self.multi_process_start = partial(run_consumer_with_multi_process,consuming_func_decorated)
+        self.multi_process_conusme = self.multi_process_start
+
+    def multi_process_start(self, process_num):
+        run_consumer_with_multi_process(self.consuming_func_decorated, process_num)
+
+    def __call__(self, *args, **kwargs):
+        return self.consuming_func_decorated(*args, **kwargs)
+
+
 def task_deco(queue_name, *, function_timeout=0,
               concurrent_num=50, specify_concurrent_pool=None, specify_async_loop=None, concurrent_mode=1,
               max_retry_times=3, log_level=10, is_print_detail_exception=True, is_show_message_get_from_broker=False,
@@ -84,7 +146,7 @@ def task_deco(queue_name, *, function_timeout=0,
               schedule_tasks_on_main_thread=False,
               function_result_status_persistance_conf=FunctionResultStatusPersistanceConfig(False, False, 7 * 24 * 3600),
               is_using_rpc_mode=False,
-              broker_kind=0):
+              broker_kind=0) -> IdeAutoCompleteHelper:
     """
     # 为了代码提示好，这里重复一次入参意义。被此装饰器装饰的函数f，函数f对象本身自动加了一些方法，例如f.push 、 f.consume等。
     :param queue_name: 队列名字。
@@ -148,6 +210,7 @@ def task_deco(queue_name, *, function_timeout=0,
         f.pub(dict(a=i, b=i * 2))
         f.push(i, i * 2)
     f.consume()
+    # f.multi_process_conusme(8)             # # 这个是新加的方法，细粒度 线程 协程并发 同时叠加8个进程，速度炸裂。主要是无需导入run_consumer_with_multi_process函数。
     # run_consumer_with_multi_process(f,8)   # 这个是细粒度 线程 协程并发 同时叠加8个进程，速度炸裂。
     '''
 
@@ -161,6 +224,7 @@ def task_deco(queue_name, *, function_timeout=0,
     for i in range(10, 20):
         consumer.publisher_of_same_queue.publish(dict(a=i, b=i * 2))
     consumer.start_consuming_message()
+    #run_consumer_with_multi_process(consumer,4) # 一次性启动4个进程。
     '''
 
     装饰器版本的 task_deco 入参 和 get_consumer 入参99%一致，唯一不同的是 装饰器版本加在了函数上自动知道消费函数了，
@@ -169,7 +233,8 @@ def task_deco(queue_name, *, function_timeout=0,
     # 装饰器版本能够自动知道消费函数，防止task_deco按照get_consumer的入参重复传参了consuming_function。
     consumer_init_params = copy.copy(locals())
 
-    def _deco(func):
+    def _deco(func) -> IdeAutoCompleteHelper:  # 加这个-> 可以实现pycahrm动态补全
+
         func.init_params = consumer_init_params
         consumer = get_consumer(consuming_function=func, **consumer_init_params)
         func.is_decorated_as_consume_function = True
@@ -180,62 +245,16 @@ def task_deco(queue_name, *, function_timeout=0,
         func.publish = func.pub = consumer.publisher_of_same_queue.publish
         func.push = func.delay = consumer.publisher_of_same_queue.push
         func.clear = func.clear_queue = consumer.publisher_of_same_queue.clear
+        func.multi_process_start = func.multi_process_conusme = partial(run_consumer_with_multi_process, func)
 
-        @wraps(func)
-        def __deco(*args, **kwargs):
-            return func(*args, **kwargs)
-
-        return __deco  # 两种方式都可以
+        # @wraps(func)
+        # def __deco(*args, **kwargs):  # 这样函数的id变化了，导致win在装饰器内部开多进程不方便。
+        #     return func(*args, **kwargs)
+        return func
+        # return __deco  # noqa # 两种方式都可以
         # return update_wrapper(__deco, func)
 
-    return _deco
-
-
-class IdeAutoCompleteHelper(LoggerMixin):
-    """
-    为了被装饰的消费函数的敲代码时候的被pycharm自动补全而写的类。
-    """
-
-    def __init__(self, consuming_func_decorated: callable):
-        """
-        :param consuming_func_decorated:   传入被task_deco装饰的函数
-
-        此框架非常非常注重，公有函数、方法、类 的名字和入参在ide开发环境下面的自动提示补全效果，如果不是为了这一点，框架能减少很多重复地方。
-        此类是防止用户调用打错字母或者不知道怎么敲代码不知道有哪些入参。所以才有这个类。
-
-        这个类是个补全类，能够使pycharm自动补全方法名字和入参。可以用，可以不用，用了后在pycharm里面补全效果会起作用。
-
-
-       from function_scheduling_distributed_framework import task_deco, IdeAutoCompleteHelper
-
-       @task_deco('queue_test_f01', qps=2, broker_kind=3)
-       def f(a, b):
-           print(f'{a} + {b} = {a + b}')
-
-
-       if __name__ == '__main__':
-           f(1000, 2000)
-           IdeAutoCompleteHelper(f).clear()  # f.clear()
-           for i in range(100, 200):
-               f.pub(dict(a=i, b=i * 2))  # f.sub方法是强行用元编程加到f上去的，是运行时状态，pycharm只能补全非运行时态的静态东西。
-               IdeAutoCompleteHelper(f).pub({'a': i * 3, 'b': i * 4})  # 和上面的发布等效，但可以自动补全方法名字和入参。
-               f.push(a=i, b=i * 2)
-               IdeAutoCompleteHelper(f).delay(i * 3,  i * 4)
-
-           IdeAutoCompleteHelper(f).start_consuming_message()  # 和 f.consume()等效
-
-        """
-        self.is_decorated_as_consume_function = consuming_func_decorated.is_decorated_as_consume_function
-
-        self.consuming_func_decorated = consuming_func_decorated
-
-        self.consumer = consuming_func_decorated.consumer  # type: AbstractConsumer
-        self.start_consuming_message = self.consume = self.start = self.consumer.start_consuming_message
-
-        self.publisher = consuming_func_decorated.publisher  # type: AbstractPublisher
-        self.publish = self.pub = self.publisher.publish
-        self.push = self.delay = self.publisher.push
-        self.clear = self.clear_queue = self.publisher.clear
+    return _deco  # noqa
 
 
 def _run_many_consumer_by_init_params(consumer_init_params_list: List[dict]):
