@@ -27,10 +27,18 @@ from threading import Lock, Thread
 import eventlet
 import gevent
 import asyncio
+
+from apscheduler.schedulers.background import BackgroundScheduler
+from apscheduler.executors.pool import ThreadPoolExecutor as ApschedulerThreadPoolExecutor
+from apscheduler.events import EVENT_JOB_MISSED
+
+from function_scheduling_distributed_framework.concurrent_pool.single_thread_executor import SoloExecutor
+from function_scheduling_distributed_framework.utils.apscheduler_monkey import patch_run_job as patch_apscheduler_run_job
+
 import pymongo
 from pymongo import IndexModel
 from pymongo.errors import PyMongoError
-from apscheduler.schedulers.background import BackgroundScheduler
+
 # noinspection PyUnresolvedReferences
 from nb_log import LoggerLevelSetterMixin, LogManager, nb_print, LoggerMixin, \
     LoggerMixinDefaultWithFileHandler, stdout_write, stderr_write, is_main_process, only_print_on_main_process
@@ -54,6 +62,8 @@ from function_scheduling_distributed_framework.utils.bulk_operation import Mongo
 from function_scheduling_distributed_framework.utils.mongo_util import MongoMixin
 from function_scheduling_distributed_framework import frame_config
 
+patch_apscheduler_run_job()
+
 
 def _delete_keys_and_return_new_dict(dictx: dict, keys: list = None):
     dict_new = copy.copy(dictx)  # 主要是去掉一级键 publish_time，浅拷贝即可。
@@ -67,7 +77,7 @@ def _delete_keys_and_return_new_dict(dictx: dict, keys: list = None):
 
 
 class ExceptionForRetry(Exception):
-    """为了重试的，抛出错误。只是定义了一个子类，用不用都可以"""
+    """为了重试的，抛出错误。只是定义了一个子类，用不用都可以，函数出任何类型错误了框架都会自动重试"""
 
 
 class ExceptionForRequeue(Exception):
@@ -432,6 +442,8 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         self.consumer_identification = f'{socket.gethostname()}_{time_util.DatetimeConverter().datetime_str.replace(":", "-")}_{os.getpid()}_{id(self)}'
 
         self._delay_task_scheduler = BackgroundScheduler(timezone=frame_config.TIMEZONE)
+        self._delay_task_scheduler.add_executor(ApschedulerThreadPoolExecutor(2))  # 只是运行submit任务到并发池，不需要很多线程。
+        self._delay_task_scheduler.add_listener(self.__apscheduler_job_miss, EVENT_JOB_MISSED)
         self._delay_task_scheduler.start()
 
         self.custom_init()
@@ -499,7 +511,8 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
             self._distributed_consumer_statistics.run()
             self.logger.warning(f'启动了分布式环境 使用 redis 的键 hearbeat:{self._queue_name} 统计活跃消费者 ，当前消费者唯一标识为 {self.consumer_identification}')
         self.keep_circulating(10, block=False)(self.check_heartbeat_and_message_count)()  # 间隔时间最好比self._unit_time_for_count小整数倍，不然日志不准。
-        self._redis_filter.delete_expire_filter_task_cycle()
+        if self._do_task_filtering:
+            self._redis_filter.delete_expire_filter_task_cycle()  # 这个默认是RedisFilter类，是个pass不运行。所以用别的消息中间件模式，不需要安装和配置redis。
         if self._schedule_tasks_on_main_thread:
             self.keep_circulating(1)(self._shedual_task)()
         else:
@@ -573,8 +586,9 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
             # noinspection PyBroadException
             t_start = time.time()
             try:
-                function_run = self.consuming_function if self._function_timeout == 0 else self._concurrent_mode_dispatcher.timeout_deco(
-                    self.__get_priority_conf(kw, 'function_timeout'))(self.consuming_function)
+                function_timeout = self.__get_priority_conf(kw, 'function_timeout')
+                function_run = self.consuming_function if not function_timeout else self._concurrent_mode_dispatcher.timeout_deco(
+                    function_timeout)(self.consuming_function)
                 if self._is_consuming_function_use_multi_params:  # 消费函数使用传统的多参数形式
                     function_result_status.result = function_run(**function_only_params)
                 else:
@@ -742,6 +756,33 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         """重新入队"""
         raise NotImplementedError
 
+    def __apscheduler_job_miss(self, event):
+        """
+        这是 apscheduler 包的事件钩子。
+        ev.function_args = job.args
+        ev.function_kwargs = job.kwargs
+        ev.function = job.func
+        :return:
+        """
+        # print(event.scheduled_run_time)
+        misfire_grace_time = self.__get_priority_conf(event.function_kwargs["kw"], 'misfire_grace_time')
+        self.logger.critical(f'现在时间是 {time_util.DatetimeConverter().datetime_str} ,'
+                             f'比此任务规定的本应该的运行时间 {event.scheduled_run_time} 相比 超过了指定的 {misfire_grace_time} 秒,放弃执行此任务 \n'
+                             f'{event.function_kwargs["kw"]["body"]} ')
+        self._confirm_consume(event.function_kwargs["kw"])
+
+        '''
+        if self.__get_priority_conf(event.function_kwargs["kw"], 'execute_delay_task_even_if_when_task_is_expired') is False:
+            self.logger.critical(f'现在时间是 {time_util.DatetimeConverter().datetime_str} ,此任务设置的延时运行已过期 \n'
+                                 f'{event.function_kwargs["kw"]["body"]} ， 此任务放弃执行')
+            self._confirm_consume(event.function_kwargs["kw"])
+        else:
+            self.logger.warning(f'现在时间是 {time_util.DatetimeConverter().datetime_str} ,此任务设置的延时运行已过期 \n'
+                                f'{event.function_kwargs["kw"]["body"]} ，'
+                                f'但框架为了防止是任务积压导致消费延后，所以仍然使其运行一次')
+            event.function(*event.function_args, **event.function_kwargs)
+        '''
+
     def _submit_task(self, kw):
         if self._judge_is_daylight():
             self._requeue(kw)
@@ -749,36 +790,32 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
             return
         publish_time = _get_publish_time(kw['body'])
         msg_expire_senconds_priority = self.__get_priority_conf(kw, 'msg_expire_senconds')
-        if msg_expire_senconds_priority != 0 and time.time() - msg_expire_senconds_priority > publish_time:
+        if msg_expire_senconds_priority and time.time() - msg_expire_senconds_priority > publish_time:
             self.logger.warning(
                 f'消息发布时戳是 {publish_time} {kw["body"].get("publish_time_format", "")},距离现在 {round(time.time() - publish_time, 4)} 秒 ,'
                 f'超过了指定的 {msg_expire_senconds_priority} 秒，丢弃任务')
             self._confirm_consume(kw)
             return 0
+
         msg_eta = self.__get_priority_conf(kw, 'eta')
         msg_countdown = self.__get_priority_conf(kw, 'countdown')
+        misfire_grace_time = self.__get_priority_conf(kw, 'misfire_grace_time')
         run_date = None
+        # print(kw)
         if msg_countdown:
-            run_date = datetime.datetime.now(tz=pytz.timezone(frame_config.TIMEZONE)) + datetime.timedelta(seconds=msg_countdown)
+            run_date = time_util.DatetimeConverter(kw['body']['extra']['publish_time']).datetime_obj + datetime.timedelta(seconds=msg_countdown)
         if msg_eta:
             run_date = time_util.DatetimeConverter(msg_eta).datetime_obj
+        # print(run_date,time_util.DatetimeConverter().datetime_obj)
+        # print(run_date.timestamp(),time_util.DatetimeConverter().datetime_obj.timestamp())
+        # print(self.concurrent_pool)
         if run_date:
             # print(repr(run_date),repr(datetime.datetime.now(tz=pytz.timezone(frame_config.TIMEZONE))))
-            if run_date.timestamp() > time.time():
-                if self._concurrent_mode == 5:  # 单线程
-                    self._delay_task_scheduler.add_job(self._run, 'date', run_date=run_date, args=(kw,))
-                else:
-                    self._delay_task_scheduler.add_job(self.concurrent_pool.submit, 'date', run_date=run_date, args=(self._run, kw))
-            else:
-                self.logger.critical(f'指定的消费时间 {run_date} 小于当前时间 {datetime.datetime.now(tz=pytz.timezone(frame_config.TIMEZONE))}')
-                self._confirm_consume(kw)
-                return 0
+            self._delay_task_scheduler.add_job(self.concurrent_pool.submit, 'date', run_date=run_date, args=(self._run,), kwargs={'kw': kw},
+                                               misfire_grace_time=misfire_grace_time)
         else:
-            if self._concurrent_mode == 5:  # 单线程
-                self._run(kw)
-            else:
-                self.concurrent_pool.submit(self._run, kw)
-        # print(self._delay_task_scheduler.get_jobs())
+            self.concurrent_pool.submit(self._run, kw)
+
         if self._is_using_distributed_frequency_control:  # 如果是需要分布式控频。
             active_num = self._distributed_consumer_statistics.active_consumer_num
             self.__frequency_control(self._qps / active_num, self._msg_schedule_time_intercal * active_num)
@@ -830,7 +867,8 @@ class ConcurrentModeDispatcher(LoggerMixin):
         if ConsumersManager.global_concurrent_mode is not None and self.consumer._concurrent_mode != ConsumersManager.global_concurrent_mode:
             ConsumersManager.show_all_consumer_info()
             # print({self.consumer._concurrent_mode, ConsumersManager.global_concurrent_mode})
-            if not {self.consumer._concurrent_mode, ConsumersManager.global_concurrent_mode}.issubset({1, 4}):
+            if not {self.consumer._concurrent_mode, ConsumersManager.global_concurrent_mode}.issubset({1, 4, 5}):
+                # threding、asyncio、solo 这几种模式可以共存。但同一个解释器不能同时选择 gevent + 其它并发模式，也不能 eventlet + 其它并发模式。
                 raise ValueError('由于猴子补丁的原因，同一解释器中不可以设置两种并发类型,请查看显示的所有消费者的信息，'
                                  '搜索 concurrent_mode 关键字，确保当前解释器内的所有消费者的并发模式只有一种')
         self._concurrent_mode = ConsumersManager.global_concurrent_mode = self.consumer._concurrent_mode
@@ -858,7 +896,8 @@ class ConcurrentModeDispatcher(LoggerMixin):
             pool_type = CustomEventletPoolExecutor
         elif self._concurrent_mode == 4:
             pool_type = AsyncPoolExecutor
-            check_not_monkey()
+        elif self._concurrent_mode == 5:
+            pool_type = SoloExecutor
         if self._concurrent_mode == 4:
             self.consumer._concurrent_pool = self.consumer._specify_concurrent_pool if self.consumer._specify_concurrent_pool is not None else pool_type(
                 self.consumer._concurrent_num, loop=self.consumer._specify_async_loop)
